open Integer
open MIPSOps
open UPP

(* ------------------------------------------------------------------------- *)

(* Determining whether an expression is pure (does not write to
   memory). This is a conservative approximation: we assume that every
   function call can write to memory. *)

let rec pure (e : expression) : bool =
  match e with
  | EConst _
  | EGetVar _ 
  | EGetGlobal _ ->
      true
  | EUnOp (_, e)
  | ELoad (e, _) ->
      pure e
  | EBinOp (_, e1, e2) ->
      pure e1 && pure e2
  | EFunCall _ ->
      false

(* ------------------------------------------------------------------------- *)

(* Integer addition and subtraction. *)

let mkadd e1 e2 =
  EBinOp (OpAdd, e1, e2)

let mksub e1 e2 =
  EBinOp (OpSub, e1, e2)

let mkneg e =
  mksub (EConst 0l) e

(* ------------------------------------------------------------------------- *)

(* Integer multiplication. *)

let rec mkmul e1 e2 =
  EBinOp (OpMul, e1, e2)

(* ------------------------------------------------------------------------- *)

(* Integer division. *)

let mkdiv e1 e2 =
  EBinOp (OpDiv, e1, e2)

(* ------------------------------------------------------------------------- *)

(* Integer comparison. *)

let rec mkcmp op e1 e2 =
  EBinOp (op, e1, e2)

(* ------------------------------------------------------------------------- *)

(* Rewriting an arbitrary operator application. This code simply
   dispatches control to one of the functions defined above. *)

let mkunop op e =
  match op with
  | PP.UOpNeg ->
      mkneg e

let mkbinop op e1 e2 =
  match op with
  | OpAdd ->
      mkadd e1 e2
  | OpSub ->
      mksub e1 e2
  | OpMul ->
      mkmul e1 e2
  | OpDiv ->
      mkdiv e1 e2
  | OpLt
  | OpLe
  | OpGt
  | OpGe
  | OpEq
  | OpNe ->
      mkcmp op e1 e2

